function K = calibCam(imgname,varargin)
% Funkcia najde kalibracnu maticu fotoaparatu. 
% imgname - obraz obsahujuci tri cierne stvorce ktore nie su navzajom
% rovnobezne
% Po zavolani funkcie sa zobrazi ciernobiely obraz, v ktorom by mali byt
% viditelne tri bele stvorce. Tie treba oznacit mysou - kliknut na stvorec.
% Vystupom funkcie je kalibracna matica K.
% Volanie: 
% calibCam(imgname) - najde kalibracu maticu. Na hladanie rohov stvorcov
% pouziva Harrisov detektor rohov.
%
% calibCam(imgname,'hough') - najde kalibracnu maticu. Na hladanie rohov
% Pouziva Houghovu transformaciu.

% Finds calibration matrix from image of three nonparallel squares.
% imgname - image with three nonparallel black squares.
% 
% After function call an image with three white squares is shown. Mark each
% of the squares using mouse click and press enter.
%
% You can call the function in two ways
% calibCam(imgname) - finds calibration matrix. Harris detector is used to
% find the corners of the squares
% calibCam(imgname,'hough') - finds calibration matrix. Hough transformation
% is used to find the corners of the squares

img = imread(imgname);

% Obraz prahujeme a v obraze vyznacme kalibracn tvorce
% We threshold the image and mark calibration squares
BW = imbinarize(rgb2gray(img));
BW = bwselect(1-BW,8);

% Njde svisl komponenty v ciernobielom obraze
% Finds connected components in black and white image
CC=bwconncomp(BW);

% Uzivatelsky definovana funkcia FindCorners najde rohy stvorca urceneho
% linearnymi indexami pixelov. Na najdenie rohov pouziva implicitne Harrisov detektor.
% Alternativne je mozne hladat rohy pomocou Houghovej transformacie - volba
% 'hough' v parametroch funkcie FindCorners.
% User defined function FindCorners finds corners of a square defined by
% its linear pixel indices. Harris corner detector is used to find corners
% of the square. Alternatively, the corners can be found using hough
% transformation - option 'hough' in function pararameters.

if nargin == 2 && all(varargin{1}=='hough')
    Pts1 = FindCorners(CC.PixelIdxList{1},CC.ImageSize,'hough');
    Pts2 = FindCorners(CC.PixelIdxList{2},CC.ImageSize,'hough');
    Pts3 = FindCorners(CC.PixelIdxList{3},CC.ImageSize,'hough');
else
    Pts1 = FindCorners(CC.PixelIdxList{1},CC.ImageSize);
    Pts2 = FindCorners(CC.PixelIdxList{2},CC.ImageSize);
    Pts3 = FindCorners(CC.PixelIdxList{3},CC.ImageSize);    
end

% Zobrazi najdene rohy v obraze.
% Shows the corner points in the image.
imshow(img);
hold on;
plot(cornerPoints(Pts1(1:2,:)'))
plot(cornerPoints(Pts2(1:2,:)'))
plot(cornerPoints(Pts3(1:2,:)'))

for k = 1:3
    plot([Pts1(1,k),Pts1(1,k+1)],[Pts1(2,k),Pts1(2,k+1)],'LineWidth',1,'Color','red')
    plot([Pts2(1,k),Pts2(1,k+1)],[Pts2(2,k),Pts2(2,k+1)],'LineWidth',1,'Color','red')
    plot([Pts3(1,k),Pts3(1,k+1)],[Pts3(2,k),Pts3(2,k+1)],'LineWidth',1,'Color','red')
end
    plot([Pts1(1,4),Pts1(1,1)],[Pts1(2,4),Pts1(2,1)],'LineWidth',1,'Color','red')
    plot([Pts2(1,4),Pts2(1,1)],[Pts2(2,4),Pts2(2,1)],'LineWidth',1,'Color','red')
    plot([Pts3(1,4),Pts3(1,1)],[Pts3(2,4),Pts3(2,1)],'LineWidth',1,'Color','red')


% Idealny (skutocny) stvorec
% Ideal (real) square
N=1;
A1 = [0,0,1]';
A2 = [0,N,1]';
A3 = [N,N,1]';
A4 = [N,0,1]';

rect = [A1,A2,A3,A4];

% Uzivatelsky definovana funkcia PtsCorTransf vrati maticu projektivnej
% transformacie, ktora vrcholy idealneho stvorca zobrazi na vrcholy
% prislusneho stvorca v obraze.
% User defined function PtsCorTransf returns a matrix of projective
% transform which maps the vertices of ideal square to vertices of a given
% square in the image.
H1 = PtsCorTransf(rect,Pts1);
H2 = PtsCorTransf(rect,Pts2);
H3 = PtsCorTransf(rect,Pts3);

% Vytvorenie rovnic na hladanie obrazu absolutnej kuzelosecky
h1 = H1(:,1);
h2 = H1(:,2);
 
M1 = [h1(1)*h2(1),h1(2)*h2(1)+h1(1)*h2(2),h1(3)*h2(1)+h1(1)*h2(3),h1(2)*h2(2),h1(3)*h2(2)+h1(2)*h2(3),h1(3)*h2(3)];
M2 = [h1(1)*h1(1),h1(2)*h1(1)+h1(1)*h1(2),h1(3)*h1(1)+h1(1)*h1(3),h1(2)*h1(2),h1(3)*h1(2)+h1(2)*h1(3),h1(3)*h1(3)]-[h2(1)*h2(1),h2(2)*h2(1)+h2(1)*h2(2),h2(3)*h2(1)+h2(1)*h2(3),h2(2)*h2(2),h2(3)*h2(2)+h2(2)*h2(3),h2(3)*h2(3)];

h1 = H2(:,1);
h2 = H2(:,2);

M3 = [h1(1)*h2(1),h1(2)*h2(1)+h1(1)*h2(2),h1(3)*h2(1)+h1(1)*h2(3),h1(2)*h2(2),h1(3)*h2(2)+h1(2)*h2(3),h1(3)*h2(3)];
M4 = [h1(1)*h1(1),h1(2)*h1(1)+h1(1)*h1(2),h1(3)*h1(1)+h1(1)*h1(3),h1(2)*h1(2),h1(3)*h1(2)+h1(2)*h1(3),h1(3)*h1(3)]-[h2(1)*h2(1),h2(2)*h2(1)+h2(1)*h2(2),h2(3)*h2(1)+h2(1)*h2(3),h2(2)*h2(2),h2(3)*h2(2)+h2(2)*h2(3),h2(3)*h2(3)];

h1 = H3(:,1);
h2 = H3(:,2);

M5 = [h1(1)*h2(1),h1(2)*h2(1)+h1(1)*h2(2),h1(3)*h2(1)+h1(1)*h2(3),h1(2)*h2(2),h1(3)*h2(2)+h1(2)*h2(3),h1(3)*h2(3)];
M6 = [h1(1)*h1(1),h1(2)*h1(1)+h1(1)*h1(2),h1(3)*h1(1)+h1(1)*h1(3),h1(2)*h1(2),h1(3)*h1(2)+h1(2)*h1(3),h1(3)*h1(3)]-[h2(1)*h2(1),h2(2)*h2(1)+h2(1)*h2(2),h2(3)*h2(1)+h2(1)*h2(3),h2(2)*h2(2),h2(3)*h2(2)+h2(2)*h2(3),h2(3)*h2(3)];

M=[M1;M2;M3;M4;M5;zeros(1,6)];

% Najdenie riesenia prislusneho systemu. Obraz absolutnej kuzelosecky je
% kuzelosecka reprezentovana maticou Om. Pat bodov urcuje kuzelosecku.
% The solution of given system of linear equations is found. The image of
% absolute conic is represented by matrix Om. The conic is determined
% by five points.

[~,~,V] = svd(M);
om = (V(:,end));
Om = [om(1),om(2),om(3);om(2),om(4),om(5);om(3),om(5),om(6)];

% K je kalibracna matica ziskana z Choleskeho rozkladu, lebo Om = inv(K*K')
% a K je horna trojuholnikova.
% The calibration matrix K is obtainHed from Cholesky decomposition, because
% Om = inv(K*K') and K is upper triangular matrix.
K = inv(chol(Om));
K=K/K(end);
hold off;
